import { log } from '@/logger/index.ts';
import { initSdk } from './init.ts';
import { notifyUrl } from '@/modules/pay-alipay-config.ts';
type payOpts = {
  /**
   * 订单号
   */
  out_trade_no: string;
  /**
   * 金额(分)
   */
  money: number;
  subject?: string;
};
/**
 *  Form页面
 *
 */
export const pay = async (opts: payOpts) => {
  const alipaySdk = initSdk();
  const money = opts?.money ? (opts.money / 100).toFixed(2) : 0.01;
  const result = alipaySdk.pageExec('alipay.trade.page.pay', 'GET', {
    bizContent: {
      out_trade_no: opts.out_trade_no,
      total_amount: money,
      subject: opts.subject,
      product_code: 'FAST_INSTANT_TRADE_PAY',
      qr_pay_mode: 4,
    },
    notify_url: notifyUrl,
  });
  console.log(result);
  return {
    form: result,
    content: {
      out_trade_no: opts.out_trade_no,
      total_amount: money,
      subject: opts.subject,
    },
  };
};

export const genOutTradeNo = () => {
  return new Date().getTime().toString();
};

export const verifySignature = (body: any) => {
  const alipaySdk = initSdk();
  const check = alipaySdk.checkNotifySign(body);
  if (!check) {
    log.error('alipay verifySignature failed', { body });
  }
  return check;
};
