import { app } from '@/app.ts';
import { genSubject } from './utils/index.ts';
import './wx-pay.ts';
import './pay.ts';
import { TradeModel } from '../trade/model.ts';
import { log } from '@/logger/index.ts';
import { PayServices } from './services/pay-services.ts';

app
  .route({
    path: 'vip',
    key: 'pay',
    description: '创建vip订单的url地址。',
    middleware: ['auth'],
  })
  .define(async (ctx) => {
    const tokenUser = ctx.state.tokenUser;
    const { level, money, payMode = 'alipay', out_trade_no, month = 1 } = ctx.query.data || {};
    const userId = tokenUser?.id;
    // 微信支付在新页面，所以，这个借口可能没有userId
    if (!level) {
      ctx.throw(400, 'level is required');
    }
    if (isNaN(Number(money))) {
      ctx.throw(400, 'money must be a number');
    }
    const subject = genSubject(level, month);
    const wrapperTrade = async (result: any, out_trade_no: string) => {
      return { ...result, out_trade_no };
    };
    const payServices = new PayServices();
    const commonData = {
      out_trade_no,
      payMode,
      money,
      subject,
    };
    const trade: TradeModel = await payServices.createTrade(
      {
        ...commonData,
        uid: userId,
        target: TradeModel.createVipTarget({ level, month }),
      },
      ctx,
    );
    commonData.out_trade_no = trade.out_trade_no;
    try {
      const result = await payServices.careatePayUrl(commonData, ctx);

      log.info(`[pay] ${userId} pay ${subject} ${money} ${trade.out_trade_no} ${trade.id} ${payMode}`);
      ctx.body = await wrapperTrade(result, trade.out_trade_no);
    } catch (error) {
      console.log('create PayUrl error', error);
      if (!out_trade_no) {
        await trade.destroy({ force: true });
      }
      ctx.throw(400, 'create url failed');
    }
  })
  .addTo(app);
