import { TradeModel, TradeTarget } from '@/routes/trade/model.ts';
import { log } from '@/logger/index.ts';
import { app } from '@/app.ts';
import { genOutTradeNo } from '../utils/index.ts';
export class PayServices {
  constructor() {}

  async careatePayUrl(
    data: {
      payMode: 'alipay' | 'wx';
      money: number;
      subject: string;
      out_trade_no: string;
    },
    ctx,
  ) {
    const { payMode, money, subject, out_trade_no } = data;
    let result;
    if (payMode === 'alipay') {
      const callResult = await app.call({
        path: 'alipay',
        key: 'pay',
        payload: {
          data: {
            money,
            subject,
            out_trade_no,
          },
        },
      });
      if (callResult.code === 200) {
        result = callResult.body;
      } else {
        log.error('pay failed', {
          message: callResult?.message,
          body: callResult?.body,
        });
        ctx.throw(400, callResult?.message || 'pay failed');
      }
    } else if (payMode === 'wx') {
      const callResult = await app.call({
        path: 'wx',
        key: 'pay',
        payload: {
          data: {
            money,
            subject,
            out_trade_no,
          },
        },
      });
      if (callResult.code === 200) {
        result = callResult.body;
      } else {
        log.error('pay failed', {
          message: callResult?.message,
          body: callResult?.body,
        });
        ctx.throw(400, callResult?.message || 'pay failed');
      }
    } else {
      log.error('payMode is Unsupported', {
        payMode,
      });
      ctx.throw(400, `current payMode ${payMode} is Unsupported`);
    }
    return result;
  }
  async checkTrade(data: { out_trade_no?: string; uid: string }, ctx) {
    if (!data.out_trade_no) {
      return null;
    }
    const { out_trade_no, uid: userId } = data;
    let trade = await TradeModel.findOne({
      where: {
        out_trade_no: out_trade_no,
      },
    });
    if (trade && trade.uid !== userId) {
      ctx.throw(403, 'trade has exist but not permission');
    }
    return trade;
  }
  /**
   * 创建订单
   * @param data
   * @param ctx
   * @returns
   */
  async createTrade(
    data: {
      /** 商户订单号 */
      out_trade_no?: string;
      /** 订单金额 */
      money: number;
      /** 订单标题 */
      subject: string;
      /** 用户id */
      uid: string;
      /** 支付方式 */
      payMode: 'alipay' | 'wx';
      /** 目标 */
      target: TradeTarget;
    },
    ctx,
  ) {
    const { money, subject, uid: userId, payMode, target } = data;
    let out_trade_no = data?.out_trade_no || genOutTradeNo();
    let trade: TradeModel = await this.checkTrade({ out_trade_no, uid: userId }, ctx);
    if (!trade) {
      trade = await TradeModel.create({
        out_trade_no: out_trade_no,
        money: money,
        status: 'WAIT_BUYER_PAY',
        subject: subject,
        uid: userId,
        type: payMode,
        data: {
          list: [],
          target: target,
        },
      });
    } else {
      trade.status = 'WAIT_BUYER_PAY';
      trade.money = money;
      trade.type = payMode;
      trade.data = {
        ...trade.data,
        target,
      };
      await trade.save({ fields: ['data', 'type', 'status', 'money'] });
    }
    return trade;
  }
}
