import { sequelize } from '@/modules/sequelize.ts';
import { DataTypes, Model } from 'sequelize';

type TradeTargetVip = {
  type: 'vip';
  level: string;
  month: number; // 月数
  updateTime: Date;
  category: 'center' | 'ai-chat';
};
type TradeTargetConvert = {
  type: 'convert';
  convertId: string;
  updateTime: Date;
};
export type TradeTarget = TradeTargetVip | TradeTargetConvert;

export type TradeData = {
  list: any[]; // 回调的信息日志
  target: TradeTarget;
};
export class TradeModel extends Model {
  declare id: string;
  /** 商户订单号 */
  declare out_trade_no: string;
  /** 订单金额 */
  declare money: number;
  /** 订单标题 */
  declare subject: string;
  /** 订单状态: WAIT_BUYER_PAY, TRADE_SUCCESS, TRADE_CLOSED, TRADE_FINISHED */
  declare status: string;
  /** 订单类型: alipay, wx */
  declare type: string;
  /** 订单数据 */
  declare data: TradeData;
  declare uid: string;

  /**
   * 创建vip目标
   * @param data
   * @returns
   */
  static createVipTarget(data: { level: string; month?: number; category?: 'center' | 'ai-chat' }) {
    const category = data.category || 'center';
    const month = data.month || 1;
    return { type: 'vip', level: data.level, month, updateTime: new Date(), category } as TradeTargetVip;
  }
}

TradeModel.init(
  {
    id: {
      type: DataTypes.UUID,
      primaryKey: true,
      defaultValue: DataTypes.UUIDV4,
    },
    out_trade_no: {
      type: DataTypes.STRING,
      allowNull: false,
      unique: true,
    },
    money: {
      type: DataTypes.INTEGER,
      allowNull: false,
    },
    subject: {
      type: DataTypes.TEXT,
      allowNull: false,
    },
    status: {
      type: DataTypes.STRING,
      allowNull: false,
      defaultValue: 'WAIT_BUYER_PAY', // WAIT_BUYER_PAY, TRADE_SUCCESS, TRADE_CLOSED, TRADE_FINISHED
    },
    type: {
      type: DataTypes.STRING,
      allowNull: false,
      defaultValue: 'alipay', // alipay, wechat
    },
    data: {
      type: DataTypes.JSONB,
      defaultValue: {
        list: [],
      },
    },
    uid: {
      type: DataTypes.UUID,
      allowNull: true,
    },
  },
  {
    sequelize,
    paranoid: true,
    modelName: 'apps_trade',
  },
);

TradeModel.sync({ alter: true, logging: false }).catch((error) => {
  console.error('sync Trade', error);
});
