import { app } from '@/app.ts';
import { log } from '@/logger/index.ts';
import { VipModel } from '../vip/models/index.ts';
import { TradeModel } from './model.ts';

app
  .route({
    path: 'trade',
    key: 'paySuccess',
  })
  .define(async (ctx) => {
    const { trade } = ctx;
    if (!trade) {
      log.error('not has trade info');
      ctx.throw('not has trade info');
    }
    const { uid, data } = trade as TradeModel;
    const { target } = data;
    if (target.type === 'vip') {
      const { level, month } = target;
      // 更新的结束时间， 添加一个月。
      let [vip, created] = await VipModel.findOrCreate({
        where: {
          userId: uid,
          category: target.category,
        },
        defaults: {
          userId: uid,
          category: target.category,
          level,
          title: target.type,
          startDate: new Date(),
          endDate: new Date(),
          month,
        },
      });
      if (created) {
        log.info('create new vip', {
          uid,
          tradeId: trade.id,
          outTradeNo: trade.out_trade_no,
          money: trade.money,
          category: target.category,
        });
      }

      vip?.addVip({
        level,
        month,
      });
    } else {
      log.error('not has vip info', {
        uid,
        tradeId: trade.id,
        outTradeNo: trade.out_trade_no,
        money: trade.money,
      });
    }
  })
  .addTo(app);

app
  .route({
    path: 'vip',
    key: 'check-pay-status',
  })
  .define(async (ctx) => {
    const { id, out_trade_no } = ctx.query.data || {};
    if (!id && !out_trade_no) {
      ctx.throw(400, 'id or out_trade_no is required');
    }
    let trade;
    if (id) {
      trade = await TradeModel.findByPk(id);
    } else {
      trade = await TradeModel.findOne({
        where: {
          out_trade_no,
        },
      });
    }
    if (!trade) {
      ctx.throw(404, 'trade not found');
    }
    ctx.body = {
      id: trade.id,
      out_trade_no: trade.out_trade_no,
      status: trade.status,
      type: trade.type,
    };
  })
  .addTo(app);
